package Plugins::Podcasts;

use Data::Dumper;
use strict;
use LWP::UserAgent;
use HTML::Entities;

eval "use XML::RSSLite;";
if ($@) {
    $Plugins::Podcasts::NoRSSLite = 1;
}

###############################################################################
sub init {
    main::plugin_register("Podcasts", "Experimental Podcast Player", "/Podcasts", \&Plugins::Podcasts::html);
    main::http_register_handler("/Podcasts/index",   \&Plugins::Podcasts::index);
    main::http_register_handler("/Podcasts/add",     \&Plugins::Podcasts::add);
}

###############################################################################
sub quit {
}

###############################################################################
sub build_zone_menu {
    my ($match) = @_;

    my $menu = "";
    foreach my $zone (keys %main::ZONES) {
        if ($zone eq $match) {
            $menu .= "<OPTION VALUE=$zone SELECTED>";
        } else {
            $menu .= "<OPTION VALUE=$zone>";
        }
        $menu .= $main::ZONES{$zone}->{ZoneName} . "\n";
    }
    return $menu
}

###############################################################################
sub html {
    my ($c, $r, $diskpath, $template) = @_;

    my %qf = $r->uri->query_form;

    $template->param("PODCASTS_ZONEMENU" => build_zone_menu($qf{zone}));

    if ($Plugins::Podcasts::NoRSSLite) {
        $template->param("PODCASTS_ERR" => 
            "<B>The perl package XML::RSSLite was not found, and probably needs to be installed.</B><P>" .
            "From a terminal or command  window please run one of the following:</BR>" .
            "&nbsp; <B>Linux/Unix:</B> perl -MCPAN -e 'install XML::RSSLite'<BR>" .
            "&nbsp; <B>Windows:</B> ppm install XML-RSSLite<BR>" .
            "&nbsp; <B>OS X:</B> /usr/local/ActivePerl-5.8/bin/ppm install XML-RSSLite<P>" .
            "Then please restart sonos.pl<P><HR>");
        return;
    } 

    my @folder_loop_data = ();
    foreach my $id (sort (keys %{$main::PREFS{Podcasts}->{casts}})) {
        my %folder_row_data;

        if (! $Plugins::Podcasts{$id}->{data}) {

            main::Log(3, "Updating " . $main::PREFS{Podcasts}->{casts}->{$id}->{url});
            $Plugins::Podcasts{$id}->{time} = time();
            my $response = $Plugins::LastFM::userAgent->get($main::PREFS{Podcasts}->{casts}->{$id}->{url});
            next if (!$response->is_success());
            $Plugins::Podcasts{$id}->{data} = $response->content;
        }

        my %result;
        parseRSS(\%result, \$Plugins::Podcasts{$id}->{data});
        $folder_row_data{FOLDER_NAME} = $result{title};
        $folder_row_data{FOLDER_DESC} = decode_entities($result{description});
        $folder_row_data{FOLDER_ID}   = $id;

        my @item_loop_data = ();
        foreach my $item (@{$result{item}}) {
            my %item_row_data;

            $item_row_data{ITEM_ZONE} = $qf{zone};
            $item_row_data{ITEM_NAME} = $item->{title};
            $item_row_data{ITEM_DESC} = decode_entities($item->{description});
            $item_row_data{ITEM_URL}  = $item->{enclosure}->{url};
            push(@item_loop_data, \%item_row_data);
        }
        $folder_row_data{PODCASTS_ITEM_LOOP} = \@item_loop_data;

        push(@folder_loop_data, \%folder_row_data);
    }

    $template->param("PODCASTS_FOLDER_LOOP" => \@folder_loop_data);
    if (defined $Plugins::Podcasts::Err) {
        $template->param("PODCASTS_ERR" => $Plugins::Podcasts::Err);
        undef $Plugins::Podcasts::Err;
    }
}

###############################################################################
sub index {
    my ($c, $r) = @_;

    my %qf = $r->uri->query_form;
    my $zonearg = "";
    if (defined $qf{zone} && $qf{zone} ne "") {
        $zonearg = "?zone=$qf{zone}";
    }
    
    if (defined $qf{refresh} && $qf{refresh} == 1) {
        foreach my $id (sort (keys %{$main::PREFS{Podcasts}->{casts}})) {
            delete $Plugins::Podcasts{$id}->{data};
        }
    }

    if (defined $qf{podcastdel} &&
        defined $main::PREFS{Podcasts}->{casts}->{$qf{podcastdel}}) {
        delete $main::PREFS{Podcasts}->{casts}->{$qf{podcastdel}};
    }
    
    if (defined $qf{podcastplay}) {
        my $url = decode_entities($qf{podcastplay});
        main::Log(3, "Play: $url on Zone: $qf{zone}");

        if  ($zonearg eq "") {
            $Plugins::Podcasts::Err = "<H1>Please select a zone.<P></H1>";
        } else  {
            main::upnp_avtransport_set_uri($qf{zone}, $url, "");
            main::upnp_avtransport_play($qf{zone});
        }
    }

    if (defined $qf{podcastadd}) {
        my $url = decode_entities($qf{podcastadd});
        main::Log(3, "Play: $url on Zone: $qf{zone}");

        if  ($zonearg eq "") {
            $Plugins::Podcasts::Err = "<H1>Please select a zone.<P></H1>";
        } else  {
            main::upnp_avtransport_add_uri($qf{zone}, $url, "");
        }
    }

    $c->send_redirect(main::http_base_url($r) . "/Podcasts/index.html$zonearg", HTTP::Status::RC_MOVED_TEMPORARILY);
    $c->force_last_request;
    return;

}

###############################################################################
sub add {
    my ($c, $r) = @_;

    my %qf = $r->uri->query_form;

    if (!defined $qf{url} || $qf{url} eq "") {
        $Plugins::Podcasts::Err = "<H1>Please enter a valid url.<P></H1>";
        $c->send_redirect(main::http_base_url($r) . "/Podcasts/index.html", HTTP::Status::RC_MOVED_TEMPORARILY);
        $c->force_last_request;
        return;
    }
    $main::PREFS{Podcasts}->{maxid}++;
    my $id  = $main::PREFS{Podcasts}->{maxid};
    $main::PREFS{Podcasts}->{casts}->{$id}->{url} = $qf{url};

    main::sonos_prefsdb_save();

    $c->send_redirect(main::http_base_url($r) . "/Podcasts/index.html", HTTP::Status::RC_MOVED_TEMPORARILY);
    $c->force_last_request;
    return;
}

1;
